'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _loadEslint = function () {
  var _ref = _asyncToGenerator(function* (options, useFlatConfig) {
    if (!useFlatConfig) {
      // Ignore any config applicable depending on the location on the filesystem
      options.useEslintrc = false;
    } else if (!loadESLint) {
      throw 'This version of ESLint does not support flat config.';
    }

    return loadESLint ? new (yield loadESLint({ useFlatConfig }))(options) : new ESLint(options);
  });

  return function _loadEslint(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var _getConfigs = function () {
  var _ref2 = _asyncToGenerator(function* (overrideConfigFile, files, useFlatConfig) {
    var esLint = yield _loadEslint({
      // Point to the particular config
      overrideConfigFile
    }, useFlatConfig);

    var configs = files.map(function () {
      var _ref3 = _asyncToGenerator(function* (filePath) {
        return (yield esLint.isPathIgnored(filePath)) ? false : esLint.calculateConfigForFile(filePath);
      });

      return function (_x6) {
        return _ref3.apply(this, arguments);
      };
    }());
    return new Set((yield Promise.all(configs)).filter(Boolean));
  });

  return function _getConfigs(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var _getConfig = function () {
  var _ref4 = _asyncToGenerator(function* (configFile, files, useFlatConfig) {
    return Array.from((yield _getConfigs(configFile, files, useFlatConfig))).reduce(function (prev, item) {
      var plugins = useFlatConfig ? Object.assign({}, prev.plugins, item.plugins) : [].concat(_toConsumableArray(new Set([].concat(prev.plugins || [], item.plugins || []))));

      return Object.assign(prev, item, {
        rules: Object.assign({}, prev.rules, item.rules),
        plugins
      });
    }, {});
  });

  return function _getConfig(_x7, _x8, _x9) {
    return _ref4.apply(this, arguments);
  };
}();

var createRuleFinder = function () {
  var _ref10 = _asyncToGenerator(function* (specifiedFile, options) {
    var configFile = _getConfigFile(specifiedFile);

    var _options$ext = options.ext,
        ext = _options$ext === undefined ? ['.js'] : _options$ext;

    var extensionRegExp = _createExtensionRegExp(ext);
    var files = glob.sync(`**/*`, { dot: true, matchBase: true }).filter(function (file) {
      return extensionRegExp.test(file);
    });

    var config = yield _getConfig(configFile, files, options.useFlatConfig);

    return new RuleFinder(config, options);
  });

  return function createRuleFinder(_x10, _x11) {
    return _ref10.apply(this, arguments);
  };
}();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var path = require('path');

var _require = require('eslint'),
    ESLint = _require.ESLint,
    Linter = _require.Linter,
    loadESLint = _require.loadESLint;

var glob = require('glob');
var difference = require('./array-diff');
var getSortedRules = require('./sort-rules');
var normalizePluginName = require('./normalize-plugin-name');

var builtinRules = void 0;
try {
  builtinRules = require('eslint/use-at-your-own-risk').builtinRules;
} catch (e) {}

function _getConfigFile(specifiedFile) {
  if (specifiedFile) {
    if (path.isAbsolute(specifiedFile)) {
      return specifiedFile;
    }
    return path.join(process.cwd(), specifiedFile);
  }
  // This is not being called with an arg. Use the package.json `main`
  return require(path.join(process.cwd(), 'package.json')).main;
}

function _getCurrentNamesRules(config) {
  return config.rules ? Object.keys(config.rules) : [];
}

function _isDeprecated(rule) {
  return rule && rule.meta && rule.meta.deprecated;
}

function _notDeprecated(rule) {
  return !_isDeprecated(rule);
}

function _getPluginRules(config, useFlatConfig) {
  var pluginRules = new Map();
  var plugins = config.plugins;
  /* istanbul ignore else */
  if (plugins) {
    if (useFlatConfig) {
      Object.entries(config.plugins).filter(function (_ref5) {
        var _ref6 = _slicedToArray(_ref5, 2),
            rules = _ref6[1].rules;

        return rules;
      }).forEach(function (_ref7) {
        var _ref8 = _slicedToArray(_ref7, 2),
            pluginName = _ref8[0],
            rules = _ref8[1].rules;

        Object.keys(rules).forEach(function (ruleName) {
          return pluginRules.set(`${pluginName}/${ruleName}`, rules[ruleName]);
        });
      });
    } else {
      plugins.forEach(function (plugin) {
        var normalized = normalizePluginName(plugin);
        var pluginConfig = require(normalized.module);
        if (pluginConfig.rules) {
          Object.keys(pluginConfig.rules).forEach(function (ruleName) {
            return pluginRules.set(`${normalized.prefix}/${ruleName}`, pluginConfig.rules[ruleName]);
          });
        }
      });
    }
  }
  return pluginRules;
}

function _getCoreRules() {
  return builtinRules || new Linter().getRules();
}

function _filterRuleNames(ruleNames, rules, predicate) {
  return ruleNames.filter(function (ruleName) {
    return predicate(rules.get(ruleName));
  });
}

function _isNotCore(rule) {
  return rule.indexOf('/') !== '-1';
}

function _escapeRegExp(str) {
  return str.replace(/[\\^$.*+?()[\]{}|]/g, '\\$&');
}

function _createExtensionRegExp(extensions) {
  var normalizedExts = extensions.map(function (ext) {
    return _escapeRegExp(ext.startsWith('.') ? ext.slice(1) : ext);
  });

  return new RegExp(`.\\.(?:${normalizedExts.join("|")})$`);
}

function RuleFinder(config, _ref9) {
  var omitCore = _ref9.omitCore,
      includeDeprecated = _ref9.includeDeprecated,
      useFlatConfig = _ref9.useFlatConfig;

  var currentRuleNames = _getCurrentNamesRules(config);
  if (omitCore) {
    currentRuleNames = currentRuleNames.filter(_isNotCore);
  }

  var pluginRules = _getPluginRules(config, useFlatConfig); // eslint-disable-line vars-on-top
  var coreRules = _getCoreRules();
  var allRules = omitCore ? pluginRules : new Map([].concat(_toConsumableArray(coreRules), _toConsumableArray(pluginRules)));

  var allRuleNames = [].concat(_toConsumableArray(allRules.keys()));
  var pluginRuleNames = [].concat(_toConsumableArray(pluginRules.keys()));
  if (!includeDeprecated) {
    allRuleNames = _filterRuleNames(allRuleNames, allRules, _notDeprecated);
    pluginRuleNames = _filterRuleNames(pluginRuleNames, pluginRules, _notDeprecated);
  }
  var deprecatedRuleNames = _filterRuleNames(currentRuleNames, allRules, _isDeprecated);
  var dedupedRuleNames = [].concat(_toConsumableArray(new Set(allRuleNames)));
  var unusedRuleNames = difference(dedupedRuleNames, currentRuleNames);

  // Get all the current rules instead of referring the extended files or documentation
  this.getCurrentRules = function () {
    return getSortedRules(currentRuleNames);
  };

  // Get all the current rules' particular configuration
  this.getCurrentRulesDetailed = function () {
    return config.rules;
  };

  // Get all the plugin rules instead of referring the extended files or documentation
  this.getPluginRules = function () {
    return getSortedRules(pluginRuleNames);
  };

  // Get all the available rules instead of referring eslint and plugin packages or documentation
  this.getAllAvailableRules = function () {
    return getSortedRules(dedupedRuleNames);
  };

  this.getUnusedRules = function () {
    return getSortedRules(unusedRuleNames);
  };

  // Get all the current rules that are deprecated
  this.getDeprecatedRules = function () {
    return getSortedRules(deprecatedRuleNames);
  };
}

module.exports = function () {
  var _ref11 = _asyncToGenerator(function* (specifiedFile) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    return createRuleFinder(specifiedFile, options);
  });

  return function (_x13) {
    return _ref11.apply(this, arguments);
  };
}();